
from __future__ import print_function
import hashlib
import os
import shutil
import tempfile

import filelock
from six.moves.urllib import request

import time
import sys


_dataset_root = os.environ.get('CHAINER_DATASET_ROOT',
                               os.path.expanduser('~/.chainer/dataset'))


def reporthook(count, block_size, total_size):
    global start_time
    if count == 0:
        start_time = time.time()
        return
    duration = time.time() - start_time
    progress_size = int(count * block_size)
    speed = int(progress_size / (1024 * duration))
    percent = int(count * block_size * 100 / total_size)
    sys.stdout.write('\r...{}, {} MB, {} KB/s, {} seconds passed'.format(
        percent, progress_size / (1024 * 1024), speed, duration))
    sys.stdout.flush()


def cached_download(url):
    """Downloads a file and caches it.

    This is different from the original ``cached_download`` in that the
    download progress is reported.

    It downloads a file from the URL if there is no corresponding cache. After
    the download, this function stores a cache to the directory under the
    dataset root (see :func:`set_dataset_root`). If there is already a cache
    for the given URL, it just returns the path to the cache without
    downloading the same file.

    Args:
        url (str): URL to download from.

    Returns:
        str: Path to the downloaded file.

    """
    cache_root = os.path.join(_dataset_root, '_dl_cache')
    try:
        os.makedirs(cache_root)
    except OSError:
        if not os.path.exists(cache_root):
            raise RuntimeError('cannot create download cache directory')

    lock_path = os.path.join(cache_root, '_dl_lock')
    urlhash = hashlib.md5(url.encode('utf-8')).hexdigest()
    cache_path = os.path.join(cache_root, urlhash)

    with filelock.FileLock(lock_path):
        if os.path.exists(cache_path):
            return cache_path

    temp_root = tempfile.mkdtemp(dir=cache_root)
    try:
        temp_path = os.path.join(temp_root, 'dl')
        print('Downloading from {}...'.format(url))
        request.urlretrieve(url, temp_path, reporthook)
        with filelock.FileLock(lock_path):
            shutil.move(temp_path, cache_path)
    finally:
        shutil.rmtree(temp_root)

    return cache_path



import collections
import numpy as np
import skimage.transform

from chainer.utils import type_check

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


class ResizeWrapper(DatasetWrapper):

    """Resize image to match a certain size.

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped.
        preprocess_idx (int or list of ints): this wrapper will preprocess k-th
            output of wrapped dataset's get_example if k is in
            `preprocess_idx`.
        output_shape: the size of output image after padding. This needs to be
            in HWC format.

    """

    def __init__(self, dataset, preprocess_idx, output_shape):
        super(ResizeWrapper, self).__init__(dataset)
        if len(output_shape) != 3:
            raise ValueError('output_shape needs to be of length 3')
        self.output_shape = output_shape
        if not isinstance(preprocess_idx, collections.Iterable):
            preprocess_idx = (preprocess_idx,)
        self.preprocess_idx = preprocess_idx

    def check_type_get_example(self, in_types):
        for idx in self.preprocess_idx:
            in_type = in_types[idx]
            type_check.expect(
                in_type.dtype.kind == 'f',
                in_type.ndim == 3
            )

    def _get_example(self, in_data):
        """Returns the i-th example.

        All returned images are in CHW format.

        Returns:
            i-th example.

        """
        out_data = list(in_data)
        for idx in self.preprocess_idx:
            img = in_data[idx]
            img = np.transpose(img, (1, 2, 0))
            scale = np.max(np.abs(img))
            out_img = skimage.transform.resize(
                img / scale, self.output_shape).astype(img.dtype)
            out_data[idx] = out_img.transpose(2, 0, 1) * scale
        return tuple(out_data)


if __name__ == '__main__':
    from chainer_cv.datasets import get_online_products
    train, test = get_online_products()
    dataset = ResizeWrapper(train, 0, (128, 128, 3))
    img = dataset.get_example(0)[0]
    img = img.transpose(1, 2, 0).astype(np.uint8)
    # you can visualize img with matplotlib



import numpy as np

import chainer


class DummyDataset(chainer.dataset.DatasetMixin):

    def __init__(self, shapes, length=100, constants=None, dtypes=None):
        self.length = length

        if not all(isinstance(elem, tuple) for elem in shapes):
            raise ValueError('`shapes` needs to be a list of tuples')
        self.shapes = shapes
        self.n_arrays = len(self.shapes)

        if constants is None:
            constants = [None] * self.n_arrays
        self.constants = constants

        if dtypes is None:
            dtypes = self.n_arrays * [np.float64]
        if len(dtypes) != self.n_arrays:
            raise ValueError('dtypes is either None or iterable whose length '
                             'is equal to shapes')
        self.dtypes = dtypes

    def __len__(self):
        return self.length

    def get_example(self, i):
        return self._get_example(i, self.shapes, self.dtypes, self.constants)

    def _get_example(self, i, shapes, dtypes, constants):
        out = []
        for shape, dtype, constant in zip(shapes, dtypes, constants):
            if constant is None:
                a = np.random.uniform(size=(shape))
            else:
                a = constant.copy()
                a = np.broadcast_to(a, shape)
            a = a.astype(dtype)
            out.append(a)
        return tuple(out)


class DummyDatasetGetRawData(DummyDataset):

    def __init__(self, shapes, get_raw_data_shapes, length=100,
                 constants=None, dtypes=None, get_raw_data_dtypes=None):
        super(DummyDatasetGetRawData, self).__init__(
            shapes, length, constants, dtypes)

        if len(get_raw_data_shapes) != self.n_arrays:
            raise ValueError('get_raw_data_shapes is either None or iterable '
                             'whose length is equal to shapes')
        self.get_raw_data_shapes = get_raw_data_shapes
        self.n_get_raw_data = len(get_raw_data_shapes)

        if len(get_raw_data_dtypes) != self.n_get_raw_data:
            raise ValueError('dtypes is either None or iterable whose length '
                             'is equal to shapes')
        self.get_raw_data_dtypes = get_raw_data_dtypes

    def get_raw_data(self, i):
        return self._get_example(i, self.get_raw_data_shapes,
                                 self.get_raw_data_dtypes, self.constants)


class SimpleDataset(chainer.dataset.DatasetMixin):

    def __init__(self, input_array):
        self.input_array = input_array

    def __len__(self):
        return len(self.input_array)

    def get_example(self, i):
        return self.input_array[i]


class ConstantReturnModel(chainer.Chain):
    """A chainer.Chain that returns constant values

    This is a ``chainer.Chain`` instance that returns `return_value` when
    a method `__call__` is called.

    """

    def __init__(self, return_value):
        super(ConstantReturnModel, self).__init__()
        self.return_value = return_value

    def __call__(self, *args):
        return self.return_value



import datasets  # NOQA
import evaluations  # NOQA
import extensions  # NOQA
import functions  # NOQA
import models  # NOQA
import training  # NOQA
import utils  # NOQA
import wrappers  # NOQA
import testing  # NOQA
import external  # NOQA



from chainer.training import extensions


class TestModeEvaluator(extensions.Evaluator):

    def evaluate(self):
        model = self.get_target('main')
        model.train = False
        ret = super(TestModeEvaluator, self).evaluate()
        model.train = True
        return ret



import copy
import numpy as np
import os.path as osp

import chainer
from chainer.dataset import convert
from chainer.utils import type_check

from chainer_cv.extensions.utils import check_type
from chainer_cv.extensions.utils import forward


class EmbedImages(chainer.training.extension.Extension):
    """An extension that embeds images collected through an iterator.

    Through iterating an iterator, images will be embedded into some low
    dimensional space given a model.

    The tuple returned by the iterator should have an image as its first index
    element. The embedded feature of the image will be saved.

    This extension will be executed in higher priority than any other default
    extensions.

    Args:
        iterator (chainer.iterators.Iterator): Dataset iterator for the
            images to be embedded. The element in the first index of the
            returned tuple will be image.
        target (chainer.Chain): model that embeds images to a feature space.
        embed_func (callable): This function will be called to embed batch
            of images.
        filename (string): NumPy array of embedded features will be saved as a
            file with this name.

    """

    invoke_before_training = False
    priority = 400  # higher than any other default priorities

    def __init__(self, iterator, target, embed_func=None,
                 filename='embed.npy'):
        self.iterator = iterator
        self.target = target
        if embed_func is None:
            embed_func = target
        self.embed_func = embed_func
        self.filename = filename

    @check_type
    def _check_type_dataset(self, in_types):
        img_type = in_types[0]
        type_check.expect(
            img_type.dtype.kind == 'f',
        )

    def __call__(self, trainer):
        iterator = copy.copy(self.iterator)
        embedded_feats = []
        for v in iterator:
            self._check_type_dataset(v[0])
            arrays = convert.concat_examples(
                v, device=chainer.cuda.get_device(self.target))
            h = forward(
                self.target, arrays[0:1], forward_func=self.embed_func)[0]
            embedded_feats.append(h)
        embedded_feats = np.concatenate(embedded_feats, axis=0)

        np.save(osp.join(trainer.out, self.filename),
                embedded_feats)



import collections
import random

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


class CropWrapper(DatasetWrapper):
    """Crop array into `cropped_shape`.

    If `start_idx` is None, this wrapper randomly crops images into one with
    shape `cropped_shape`.
    If `start_idx` is not None, returned images will be something like
    `image[start_idx[0]:start_idx[0] + cropped_shape[0], ...]`.

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped.
        augment_idx (int or list of ints): this wrapper will augment k-th
            output of wrapped dataset's get_example if k is in `augment_idx`.
        cropped_shape (tuple of ints): shape of data after cropping.
        start_idx (tuple of ints): If this is None, this wrapper randomly
            crops images.

    """

    def __init__(self, dataset, augment_idx, cropped_shape, start_idx=None):
        super(CropWrapper, self).__init__(dataset)

        self.cropped_shape = cropped_shape

        if not isinstance(augment_idx, collections.Iterable):
            augment_idx = (augment_idx,)
        self.augment_idx = augment_idx

        self.start_idx = start_idx

    def _get_example(self, in_data):
        """Returns the i-th example.

        All returned images are in CHW format.

        Args:
            in_data (tuple): The i-th example from the wrapped dataset.

        Returns:
            i-th example.

        """
        out_data = list(in_data)
        for idx in self.augment_idx:
            img = in_data[idx]
            shape = img.shape
            slices = []
            for i, (width, cropped_width) in enumerate(
                    zip(shape, self.cropped_shape)):
                if self.start_idx is None:
                    if width > cropped_width:
                        start_idx = random.choice(range(width - cropped_width))
                    elif width == cropped_width:
                        start_idx = 0
                    else:
                        raise ValueError('width of an input has to be larger '
                                         'than values in cropped_shape')
                else:
                    start_idx = self.start_idx[i]
                slices.append(slice(start_idx, start_idx + cropped_width))
            out_data[idx] = img[tuple(slices)]
        return tuple(out_data)



import numpy as np
import os.path as osp
import tempfile

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


class CacheArrayDatasetWrapper(DatasetWrapper):
    """This caches outputs from wrapped dataset and reuse them.

    Note that it converts outputs from wrapped dataset into numpy.ndarray.
    This only works when the wrapped dataset returns arrays whose shapes are
    same.

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped
        copy (bool): If `copy` is True, this self.get_example returns a copy of
            cached array.

    """

    def __init__(self, dataset, copy=True):
        super(CacheArrayDatasetWrapper, self).__init__(dataset)
        self.initialized = False
        self.cache = None
        self.has_cache = [False] * len(self._dataset)
        self.copy = copy

    def get_example(self, i):
        """Returns the i-th example.

        This caches the requested example if it has not been already cached.
        Once cached, the __getitem__ method of the wrapped dataset will not be
        called. Instead, the cached data will be loaded.

        Args:
            i (int): The index of the example.

        Returns:
            i-th example

        """
        if not self.initialized:
            self._initialize(i)
            self.initialized = True

        if not self.has_cache[i]:
            arrays = self._dataset[i]
            for arr_i, a in enumerate(arrays):
                self.cache[arr_i][i] = np.array(a)
            self.has_cache[i] = True
        if self.copy:
            out = tuple([a_cache[i].copy() for a_cache in self.cache])
        else:
            out = tuple([a_cache[i] for a_cache in self.cache])
        return out

    def _initialize(self, i):
        arrays = self._dataset[i]
        self.cache = [None] * len(arrays)
        for arr_i, a in enumerate(arrays):
            if not isinstance(np.array(a), np.ndarray):
                raise ValueError(
                    'The dataset wrapped by CacheDatasetWrapper needs to '
                    'return tuple of numpy.ndarray')
            shape = (len(self),) + a.shape
            filename = osp.join(
                tempfile.mkdtemp(), 'cache_{}.data'.format(arr_i))
            self.cache[arr_i] = np.memmap(
                filename, dtype=a.dtype, mode='w+', shape=shape)


if __name__ == '__main__':
    import chainer
    train, test = chainer.datasets.get_mnist()
    cached_dataset = CacheArrayDatasetWrapper(train)



import collections
import random

from chainer.utils import type_check

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


class RandomMirrorWrapper(DatasetWrapper):
    """Crop array by `crop_width`  along each dimension.

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped.
        augment_idx (int or list of ints): this wrapper will augment k-th
            output of wrapped dataset's get_example if k is in `augment_idx`.
        orientation ({'h', 'v', 'both'}): chooses whether to mirror
            horizontally or vertically.
    """

    def __init__(self, dataset, augment_idx, orientation='h'):
        super(RandomMirrorWrapper, self).__init__(dataset)

        if orientation not in ['h', 'v', 'both']:
            raise ValueError('orientation has to be either \'h\', \'v\' or '
                             '\'both\'')
        if orientation == 'both':
            orientation = ['h', 'v']
        else:
            orientation = list(orientation)
        self.orientation = orientation

        if not isinstance(augment_idx, collections.Iterable):
            augment_idx = (augment_idx,)
        self.augment_idx = augment_idx

    def check_type_get_example(self, in_types):
        for idx in self.augment_idx:
            in_type = in_types[idx]
            type_check.expect(
                in_type.ndim == 3
            )

    def _get_example(self, in_data):
        """Returns the i-th example.

        All returned images are in CHW format.

        Args:
            i (int): The index of the example.

        Returns:
            i-th example.

        """
        out_data = list(in_data)
        h_mirror = random.choice([True, False])
        v_mirror = random.choice([True, False])
        for idx in self.augment_idx:
            img = in_data[idx]
            if 'h' in self.orientation:
                if h_mirror:
                    img = img[:, :, ::-1]
            if 'v' in self.orientation:
                if v_mirror:
                    img = img[:, ::-1, :]
            out_data[idx] = img
        return tuple(out_data)



import collections
import numpy as np
import os.path as osp
from skimage.color import label2rgb
import warnings

import chainer
from chainer.utils import type_check

from chainer_cv.extensions.utils import check_type
from chainer_cv.extensions.utils import forward

try:
    from matplotlib import pyplot as plt
    _available = True

except ImportError:
    _available = False


def _check_available():
    if not _available:
        warnings.warn('matplotlib is not installed on your environment, '
                      'so nothing will be plotted at this time. '
                      'Please install matplotlib to plot figures.\n\n'
                      '  $ pip install matplotlib\n')


class SemanticSegmentationVisOut(chainer.training.extension.Extension):
    """An extension that visualizes input and output of semantic segmentation

    This extension visualizes predicted label, ground truth label and input
    image.

    The model that is used for forwarding is obtained by the command below.
    model = trainer.updater.get_optimizer('main').target

    Args:
        indices (list of ints or int): List of indices for data to be
            visualized
        target: Link object used for visualization
        dataset: Dataset class that produces inputs to ``target``.
        n_class (int): number of classes
        filename_base (int): basename for saved image
        forward_func (callable): Callable that is used to forward data input.
            This callable takes all the arrays returned by the dataset as
            input. Also, this callable returns an prediction of labels.
            If `forward_func = None`, then the model's `__call__` method will
            be called.

    """
    invoke_before_training = False

    def __init__(self, indices, dataset, target, n_class,
                 filename_base='semantic_seg', forward_func=None):
        _check_available()
        if not _available:
            return

        if not isinstance(indices, collections.Iterable):
            indices = list(indices)
        self.dataset = dataset
        self.target = target
        self.indices = indices
        self.n_class = n_class
        self.filename_base = filename_base
        self.forward_func = forward_func

    @check_type
    def _check_type_dataset(self, in_types):
        img_type = in_types[0]
        label_type = in_types[1]

        type_check.expect(
            img_type.dtype.kind == 'f',
            label_type.dtype.kind == 'i',
            img_type.shape[0] == 3,
            label_type.shape[0] == 1,
            img_type.shape[1] == label_type.shape[1],
            img_type.shape[2] == label_type.shape[2],
            img_type.ndim == 3,
            label_type.ndim == 3
        )

    @check_type
    def _check_type_model(self, in_types):
        predict_type = in_types[0]
        type_check.expect(
            predict_type.ndim == 4,
            predict_type.shape[0] == 1,
            predict_type.shape[1] == self.n_class
        )

    @check_type
    def _check_type_get_raw_data(self, in_types):
        img_type = in_types[0]
        label_type = in_types[1]
        type_check.expect(
            img_type.ndim == 3,
            label_type.ndim == 2,
            img_type.shape[2] == 3
        )

    def __call__(self, trainer):
        if not _available:
            return

        for idx in self.indices:
            formated_filename_base = osp.join(trainer.out, self.filename_base)
            out_file = (formated_filename_base +
                        '_idx={}'.format(idx) +
                        '_iter={}'.format(trainer.updater.iteration) + '.jpg')

            inputs = self.dataset[idx]
            gt = inputs[1]
            self._check_type_dataset(inputs)
            out = forward(self.target, inputs,
                          forward_func=self.forward_func, expand_dim=True)
            self._check_type_model(out)
            label = np.argmax(out[0][0], axis=0)

            if not hasattr(self.dataset, 'get_raw_data'):
                raise ValueError(
                    'the dataset class needs to have a method '
                    '``get_raw_data`` for a visualization extension')
            raw_inputs = self.dataset.get_raw_data(idx)
            self._check_type_get_raw_data(raw_inputs)
            vis_img = raw_inputs[0]

            # mask
            label[gt[0] == -1] = -1

            # prepare label
            x_slices, y_slices = _get_pad_slices(gt[0], unknown_val=-1)
            label = _process_label(label, self.n_class)
            gt_label = _process_label(gt[0], self.n_class)
            label = label[y_slices, x_slices]
            gt_label = gt_label[y_slices, x_slices]

            plt.close()
            plt.subplot(2, 2, 1)
            plt.imshow(vis_img)
            plt.axis('off')
            plt.subplot(2, 2, 3)
            plt.imshow(label, vmin=-1, vmax=21)
            plt.axis('off')
            plt.subplot(2, 2, 4)
            plt.imshow(gt_label, vmin=-1, vmax=21)
            plt.axis('off')
            plt.savefig(out_file)


def bitget(byteval, idx):
    return ((byteval & (1 << idx)) != 0)


def labelcolormap(N=256):
    cmap = np.zeros((N, 3))
    for i in xrange(0, N):
        id = i
        r, g, b = 0, 0, 0
        for j in xrange(0, 8):
            r = np.bitwise_or(r, (bitget(id, 0) << 7 - j))
            g = np.bitwise_or(g, (bitget(id, 1) << 7 - j))
            b = np.bitwise_or(b, (bitget(id, 2) << 7 - j))
            id = (id >> 3)
        cmap[i, 0] = r
        cmap[i, 1] = g
        cmap[i, 2] = b
    cmap = cmap.astype(np.float32) / 255
    return cmap


def _process_label(label, n_class, bg_label=0):
    colors = labelcolormap(n_class)
    label_viz = label2rgb(
        label, image=None, colors=colors[1:], bg_label=bg_label)
    # label 0 color: (0, 0, 0, 0) -> (0, 0, 0, 255)
    label_viz[label == 0] = 0
    return label_viz


def _get_pad_slices(label, unknown_val=-1):
    where_val = np.where(label != -1)
    y = where_val[0]
    x = where_val[1]
    y_slices = slice(np.min(y), np.max(y) + 1, None)
    x_slices = slice(np.min(x), np.max(x) + 1, None)
    return x_slices, y_slices



import os.path as osp
import tempfile
import shelve

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


class CacheDatasetWrapper(DatasetWrapper):
    """This caches outputs from wrapped dataset and reuse them.

    Note that it converts outputs from wrapped dataset into numpy.ndarray.
    Unlike `CacheArrayDatasetWrapper`, this works even in the case when
    the wrapped dataset returns arrays whose shapes are not same.

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped
        copy (bool): If `copy` is True, this self.get_example returns a copy of
            cached array.

    """

    def __init__(self, dataset, copy=True):
        super(CacheDatasetWrapper, self).__init__(dataset)
        self.cache = None
        self._initialize()

    def _initialize(self):
        filename = osp.join(tempfile.mkdtemp(), 'cache.db')
        self.cache = shelve.open(filename, protocol=2)

    def get_example(self, i):
        """Returns the i-th example.

        This caches the requested example if it has not been already cached.
        Once cached, the __getitem__ method of the wrapped dataset will not be
        called. Instead, the cached data will be loaded.

        Args:
            i (int): The index of the example.

        Returns:
            i-th example

        """
        key = str(i)
        if key not in self.cache:
            self.cache[key] = self._dataset[i]
        return self.cache[key]

    def __del__(self):
        if self.cache is not None:
            self.cache.close()


if __name__ == '__main__':
    import chainer
    train, test = chainer.datasets.get_mnist()
    cached_dataset = CacheDatasetWrapper(train)



from semantic_segmentation.pascal_voc_dataset import PascalVOCDataset  # NOQA
from image_retrieval.online_products_dataset import OnlineProductsDataset  # NOQA
from image_retrieval.online_products_dataset import get_online_products  # NOQA



import copy
import numpy as np
import os.path as osp
from skimage.io import imread
from skimage.color import gray2rgb
import zipfile

import chainer
from chainer.dataset import download

from chainer_cv import utils
from chainer_cv.wrappers import KeepSubsetWrapper


root = 'pfnet/chainer_cv/online_products'
url = 'ftp://cs.stanford.edu/cs/cvgl/Stanford_Online_Products.zip'


def _get_online_products():
    data_root = download.get_dataset_directory(root)
    base_path = osp.join(data_root, 'Stanford_Online_Products')
    if osp.exists(base_path):
        # skip downloading
        return base_path

    download_file_path = utils.cached_download(url)

    with zipfile.ZipFile(download_file_path, 'r') as z:
        z.extractall(data_root)
    return base_path


class OnlineProductsDataset(chainer.dataset.DatasetMixin):

    """Simple class to load data from Online Products Dataset [1].

    .. [1] Stanford Online Products dataset
        http://cvgl.stanford.edu/projects/lifted_struct

    All returned images are in RGB format.

    Args:
        data_dir (string): Path to the root of the training data. If this is
            'auto', this class will automatically download data for you
            under ``$CHAINER_DATASET_ROOT/pfnet/chainer_cv/pascal_voc``.
    """

    def __init__(self, base_dir='auto'):
        if base_dir == 'auto':
            base_dir = _get_online_products()
        self.base_dir = base_dir

        self.class_ids = []
        self.super_class_ids = []
        self.paths = []
        for mode in ['train', 'test']:
            id_list_file = osp.join(base_dir, 'Ebay_{}.txt'.format(mode))
            ids_tmp = [id_.strip().split() for id_ in open(id_list_file)][1:]
            self.class_ids += [int(id_[1]) for id_ in ids_tmp]
            self.super_class_ids += [int(id_[2]) for id_ in ids_tmp]
            self.paths += [osp.join(base_dir, id_[3]) for id_ in ids_tmp]

        self.class_ids_dict = self._list_to_dict(self.class_ids)
        self.super_class_ids_dict = self._list_to_dict(self.super_class_ids)

    def _list_to_dict(self, l):
        dict_ = {}
        for i, v in enumerate(l):
            if v not in dict_:
                dict_[v] = []
            dict_[v].append(i)
        return dict_

    def __len__(self):
        return len(self.paths)

    def get_example(self, i):
        """Returns the i-th example.

        Returns a color image, class_id and super_class_id. The image is in CHW
        format.

        Args:
            i (int): The index of the example.
        Returns:
            i-th example
        """

        class_id = np.array(self.class_ids[i], np.int32)
        super_class_id = np.array(self.super_class_ids[i], np.int32)

        img = imread(self.paths[i])

        if img.ndim == 2:
            img = gray2rgb(img)
        img = img.transpose(2, 0, 1).astype(np.float32)
        return img, class_id, super_class_id

    def get_raw_data(self, i):
        """Returns the i-th example's image and class data in HWC format.

        The color image that is returned is RGB.

        Args:
            i (int): The index of the example.

        Returns:
            i-th example (image, class_id, super_class_id)

        """
        img = imread(self.paths[i])
        class_id = self.class_ids[i]
        super_class_id = self.super_class_ids[i]
        return img, class_id, super_class_id

    def get_ids(self, class_id):
        """Get indices of examples in the given class.

        Args:
            class_id (int): the class id.

        Returns:
            list of indices of examples whose class ids are `class_id`.

        """
        return copy.copy(self.class_ids_dict[class_id])


def get_online_products(base_dir='auto',
                        train_classes=None, test_classes=None):
    """Gets the Online Products Dataset.

    This method returns train and test split of Online Products Dataset as
    done in [1].
    It uses the first 11318 classes for training and remaining 11316 classes
    for testing.

    .. [1] Deep Metric Learning via Lifted Structured Feature Embedding
        https://arxiv.org/abs/1511.06452

    Args:
        data_dir (string): Path to the root of the training data. If this is
            'auto', this class will automatically download data for you
            under ``$CHAINER_DATASET_ROOT/pfnet/chainer_cv/pascal_voc``.
        train_classes (list of int): The train dataset will contain images
            whose class ids are included in `train_classes`. If this is
            `None`, the first 11318 classes are used as done in [1].
        test_classes (list of int): The test dataset will contain images
            whose class ids are included in `test_classes`. If this is
            `None`, the last 11316 classes are used as done in [1].

    Returns:
        A tuple of two datasets
        `chainer_cv.datasets.image_retrieval.OnlineProductsDataset` which
        are wrapped by `chainer_cv.wrappers.KeepSubsetWrapper`.
        The first dataset is for training and the second is for testing.

    """
    if train_classes is None:
        train_classes = range(1, 11319)
    if test_classes is None:
        test_classes = range(11319, 22634 + 1)
    dataset = OnlineProductsDataset(base_dir)

    train_ids = []
    for i in train_classes:
        train_ids += dataset.get_ids(i)
    test_ids = []
    for i in test_classes:
        test_ids += dataset.get_ids(i)

    train_dataset = KeepSubsetWrapper(
        dataset, train_ids, wrapped_func_names=['get_example', 'get_raw_data'])
    test_dataset = KeepSubsetWrapper(
        dataset, test_ids, wrapped_func_names=['get_example', 'get_raw_data'])
    return train_dataset, test_dataset


if __name__ == '__main__':
    train, test = get_online_products()



import six

import numpy as np

import chainer
from chainer.utils import type_check


class DatasetWrapper(chainer.dataset.DatasetMixin):
    """Wrap dataset class to add functionalities.

    This class is wrapped around a dataset class or another wrapper to add a
    functionality.

    The method `_get_example` should contain codes that are necessary to add
    functionality.

    If an output of the wrapped dataset is not tuple, the wrapper class forces
    it to be a tuple when passed to `_get_example`. In that case, the final
    value that is returned by `get_example` is forced back to non-tuple.

    """

    def __init__(self, dataset):
        self._dataset = dataset
        self._update_wrapper_stack()

    def _update_wrapper_stack(self):
        """Keep a list of all the wrappers that have been appended to the stack.

        """

        self._wrapper_stack = getattr(self._dataset, '_wrapper_stack', [])
        self._wrapper_stack.append(self)

    def __len__(self):
        return len(self._dataset)

    def __getattr__(self, name):
        if name == 'get_example':
            return self.get_example
        elif name == '__getitem__':
            return self.__getitem__
        orig_attr = getattr(self._dataset, name)
        return orig_attr

    def get_example(self, i):
        """Returns the i-th example.

        Args:
            i (int): The index of the example.

        Returns:
            The i-th example.

        """
        # this can be overridden
        in_data = self._dataset[i]

        # check if input is tuple
        converted_tuple = False
        if not isinstance(in_data, tuple):
            in_data = (in_data,)
            converted_tuple = True

        # check type
        self._check_data_type_get_example(in_data)

        # convert back to non tuple if necessary
        out = self._get_example(in_data)
        if converted_tuple:
            out = out[0]
        return out

    def _get_example(self, in_data):
        """Returns the i-th example given values from the wrapped dataset.

        Args:
            in_data (tuple): The i-th example from the wrapped dataset.

        Returns:
            The i-th example.

        """
        return in_data

    def _check_data_type_get_example(self, in_data):
        in_data = tuple([np.array(v) for v in in_data])
        in_type = type_check.get_types(in_data, 'in_types', False)
        try:
            self.check_type_get_example(in_type)
        except type_check.InvalidType as e:
            msg = """
Invalid operation is performed in: {0} (get_example)

{1}""".format(self.label, str(e))
            six.raise_from(
                type_check.InvalidType(e.expect, e.actual, msg=msg), None)

    def check_type_get_example(self, in_types):
        """Checks types of input data before forward propagation.

        Before :meth:`get_example` is called, this function is called.
        You need to validate types of input data in this function
        using :ref:`the type checking utilities <type-check-utils>`.

        Args:
            in_types (~chainer.utils.type_check.TypeInfoTuple): The type
                information of input data for :meth:`get_example`.
        """
        pass

    def __str__(self):
        return '<{}{}>'.format(type(self).__name__, self._dataset)

    def __repr__(self):
        return str(self)



import numpy as np

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


class PadWrapper(DatasetWrapper):

    """Pads image to fill given image size

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped
        max_size: the size of output image after padding

    """

    def __init__(self, dataset, max_size=(512, 512)):
        super(PadWrapper, self).__init__(dataset)
        self.max_size = max_size

    def _get_example(self, in_data):
        """Returns the i-th example.

        Args:
            i (int): The index of the example.

        Returns:
            i-th example.

        """
        img, label = in_data

        x_slices, y_slices = self._get_pad_slices(img, max_size=self.max_size)
        out = np.zeros((3,) + self.max_size, dtype=np.float32)
        # unknown is -1
        out_label = np.ones((1,) + self.max_size, dtype=np.int32) * -1
        out[:, y_slices, x_slices] = img
        out_label[:, y_slices, x_slices] = label
        return out, out_label

    def _get_pad_slices(self, img, max_size=(500, 500)):
        _, H, W = img.shape

        if H < max_size[0]:
            diff_y = max_size[0] - H
            margin_y = diff_y / 2
            if diff_y % 2 == 0:
                y_slices = slice(margin_y, max_size[0] - margin_y)
            else:
                y_slices = slice(margin_y, max_size[0] - margin_y - 1)
        else:
            y_slices = slice(0, max_size[0])

        if W < max_size[1]:
            diff_x = max_size[1] - W
            margin_x = diff_x / 2
            if diff_x % 2 == 0:
                x_slices = slice(margin_x, max_size[1] - margin_x)
            else:
                x_slices = slice(margin_x, max_size[1] - margin_x - 1)
        else:
            x_slices = slice(0, max_size[1])
        return x_slices, y_slices


if __name__ == '__main__':
    from chainer_cv.datasets.pascal_voc_dataset import PascalVOCDataset
    import os.path as osp
    base_dir = osp.expanduser('~/datasets/VOC2012')
    dataset = PadWrapper(PascalVOCDataset(base_dir))

    img, label = dataset.get_example(0)



import numpy as np
import six

from chainer.utils import type_check
import chainer


def forward(model, inputs, forward_func=None, expand_dim=False):
    """Forward model with given inputs

    Args:
        model (chainer.Chain):
            If model stores its paramters in a GPU, the GPU will be used
            for forwarding.
        inputs: tuple of numpy.ndarray to be used as input. If `expand_dim`
            is True, the first axis will be added.
        forward_func (callable): called to forward
        expand_dim (bool)

    """

    if forward_func is None:
        forward_func = model
    input_vars = []
    outputs = []
    xp = model.xp
    for a in inputs:
        if not isinstance(a, np.ndarray):
            raise ValueError('input has to be ndarray')
        if expand_dim:
            a = np.expand_dims(a, axis=0)

        a_var = chainer.Variable(a)
        if xp != np:
            a_var.to_gpu()
        input_vars.append(a_var)

    output_vars = forward_func(*input_vars)
    if not isinstance(output_vars, tuple):
        output_vars = (output_vars,)
    for out in output_vars:
        if isinstance(out, chainer.Variable):
            out = out.data
        out = chainer.cuda.to_cpu(out)
        outputs.append(out)
    return tuple(outputs)


def check_type(check_type_func, name=None):
    """
    This is a decorator for a class method.
    """

    def wrapper(self, in_data):
        if any([not isinstance(in_data_i, np.ndarray) and
                not isinstance(in_data_i, chainer.cuda.ndarray) for
                in_data_i in in_data]):
            in_data_tmp = list(in_data)
            for i, in_data_i in enumerate(in_data):
                if (not isinstance(in_data_i, np.ndarray) and
                        not isinstance(in_data_i, chainer.cuda.ndarray)):
                    in_data_tmp[i] = np.array(in_data_i)
            in_data_tmp = tuple(in_data_tmp)
        else:
            in_data_tmp = in_data

        in_types = type_check.get_types(in_data_tmp, 'in_types', False)
        try:
            check_type_func(self, in_types)
        except type_check.InvalidType as e:
            msg = """
    Invalid operation is performed in: {0}

    {1}""".format(name, str(e))
            six.raise_from(
                type_check.InvalidType(e.expect, e.actual, msg=msg), None)
    return wrapper



import numpy as np
import os.path as osp
from PIL import Image
from skimage.io import imread
import tarfile

import chainer
from chainer.dataset import download

from chainer_cv import utils


root = 'pfnet/chainer_cv/pascal_voc'
url = 'http://host.robots.ox.ac.uk/pascal/VOC/voc2012/' \
    'VOCtrainval_11-May-2012.tar'


def _get_pascal_voc():
    data_root = download.get_dataset_directory(root)
    base_path = osp.join(data_root, 'VOCdevkit/VOC2012')
    if osp.exists(base_path):
        # skip downloading
        return base_path

    download_file_path = utils.cached_download(url)

    with tarfile.TarFile(download_file_path, 'r') as t:
        t.extractall(data_root)
    return base_path


class PascalVOCDataset(chainer.dataset.DatasetMixin):

    """Simple class to load data from Pascal VOC2011 for semantic segmentation

    Args:
        data_dir (string): Path to the root of the training data. If this is
            'auto', this class will automatically download data for you
            under ``$CHAINER_DATASET_ROOT/pfnet/chainer_cv/pascal_voc``.
        bgr (bool): If true, method `get_example` will return an image in BGR
            format.
    """

    target_names = np.array([
        'background',
        'aeroplane',
        'bicycle',
        'bird',
        'boat',
        'bottle',
        'bus',
        'car',
        'cat',
        'chair',
        'cow',
        'diningtable',
        'dog',
        'horse',
        'motorbike',
        'person',
        'potted plant',
        'sheep',
        'sofa',
        'train',
        'tv/monitor',
    ])

    def __init__(self, base_dir='auto', mode='train', bgr=True):
        if mode not in ['train', 'trainval', 'val']:
            raise ValueError(
                'please pick mode from \'train\', \'trainval\', \'val\'')

        if base_dir == 'auto':
            base_dir = _get_pascal_voc()

        id_list_file = osp.join(
            base_dir, 'ImageSets/Segmentation/{0}.txt'.format(mode))
        self.ids = [id_.strip() for id_ in open(id_list_file)]

        self.base_dir = base_dir
        self.bgr = bgr

    def __len__(self):
        return len(self.ids)

    def get_example(self, i):
        """Returns the i-th example.

        Returns a color image and a label image. Both of them are in CHW
        format.

        Args:
            i (int): The index of the example.

        Returns:
            tuple of color image and label whose shapes are (3, H, W) and \
                (1, H, W) respectively. H and W are height and width of the \
                images. The dtype of the color image is ``numpy.float32`` and \
                the dtype of the label image is ``numpy.int32``. The color \
                channel of the color image is determined by the paramter \
                ``self.bgr``.

        """
        if i >= len(self):
            raise IndexError('index is too large')
        img, label = self.get_raw_data(i)
        if self.bgr:
            img = img[:, :, ::-1]
        img = img.transpose(2, 0, 1).astype(np.float32)
        label = label[None]
        return img, label

    def get_raw_data(self, i):
        """Returns the i-th example's images in HWC format.

        The color image that is returned is RGB.

        Args:
            i (int): The index of the example.

        Returns:
            i-th example (image, label image)

        """
        img_file = osp.join(self.base_dir, 'JPEGImages', self.ids[i] + '.jpg')
        img = imread(img_file, mode='RGB')
        label = self._load_label(self.base_dir, self.ids[i])
        return img, label

    def _load_label(self, base_dir, id_):
        label_rgb_file = osp.join(
            base_dir, 'SegmentationClass', id_ + '.png')
        im = Image.open(label_rgb_file)
        label = np.array(im, dtype=np.uint8).astype(np.int32)
        label[label == 255] = -1
        return label


if __name__ == '__main__':
    dataset = PascalVOCDataset()
    for i in range(100):
        dataset.get_example(i)



from collections import OrderedDict
import os

from chainer.dataset import download
from chainer.functions.activation.relu import relu
from chainer.functions.noise.dropout import dropout
from chainer.functions.pooling.max_pooling_2d import max_pooling_2d
from chainer.functions.pooling.average_pooling_2d import average_pooling_2d
from chainer.functions.normalization.local_response_normalization\
    import local_response_normalization
from chainer import link
from chainer.links.connection.convolution_2d import Convolution2D
from chainer.links.connection.inception import Inception
from chainer.links.connection.linear import Linear
from chainer.serializers import npz


class GoogLeNet(link.Chain):
    """A pre-trained CNN model of GoogLeNet [1].

    During initialization, this chain model automatically downloads
    the pre-trained caffemodel, convert to another chainer model,
    stores it on your local directory, and initializes all the parameters
    with it. This model would be useful when you want to extract a semantic
    feature vector from a given image, or fine-tune the model
    on a different dataset.
    Note that this pre-trained model is released under Creative Commons
    Attribution License.

    If you want to manually convert the pre-trained caffemodel to a chainer
    model that can be specified in the constructor,
    please use ``convert_caffemodel_to_npz`` classmethod instead.

    .. [1] K. Simonyan and A. Zisserman, `Going Deeper with Convolutions
        <https://arxiv.org/abs/1409.4842>`_

    Args:
        pretrained_model (str): the destination of the pre-trained
            chainer model serialized as a ``.npz`` file.
            If this argument is specified as ``auto``,
            it automatically loads and converts the caffemodel from
            ``$CHAINER_DATASET_ROOT/pfnet/chainer/models/
            bvlc_googlenet.caffemodel``,
            where ``$CHAINER_DATASET_ROOT`` is set as
            ``$HOME/.chainer/dataset`` unless you specify another value
            as an environment variable. Note that in this case the converted
            chainer model is stored on the same directory and automatically
            used from the second time.
            If the argument is specified as ``None``, all the parameters
            are not initialized by the pre-trained model.

    Attributes:
        available_layers (list of str): The list of available layer names
            used by ``__call__`` and ``extract`` methods.
    """

    def __init__(self, pretrained_model='auto'):
        super(GoogLeNet, self).__init__(
            conv1=Convolution2D(3,  64, 7, stride=2, pad=3),
            conv2_reduce=Convolution2D(64,  64, 1),
            conv2=Convolution2D(64, 192, 3, stride=1, pad=1),
            inc3a=Inception(192,  64,  96, 128, 16,  32,  32),
            inc3b=Inception(256, 128, 128, 192, 32,  96,  64),
            inc4a=Inception(480, 192,  96, 208, 16,  48,  64),
            inc4b=Inception(512, 160, 112, 224, 24,  64,  64),
            inc4c=Inception(512, 128, 128, 256, 24,  64,  64),
            inc4d=Inception(512, 112, 144, 288, 32,  64,  64),
            inc4e=Inception(528, 256, 160, 320, 32, 128, 128),
            inc5a=Inception(832, 256, 160, 320, 32, 128, 128),
            inc5b=Inception(832, 384, 192, 384, 48, 128, 128),
            loss3_fc=Linear(1024, 1000),

            loss1_conv=Convolution2D(512, 128, 1),
            loss1_fc1=Linear(4 * 4 * 128, 1024),
            loss1_fc2=Linear(1024, 1000),

            loss2_conv=Convolution2D(528, 128, 1),
            loss2_fc1=Linear(4 * 4 * 128, 1024),
            loss2_fc2=Linear(1024, 1000),
        )
        if pretrained_model == 'auto':
            _retrieve(
                'bvlc_googlenet.npz',
                'http://dl.caffe.berkeleyvision.org/bvlc_googlenet.caffemodel',
                self)
        elif pretrained_model:
            npz.load_npz(pretrained_model, self)
        self.functions = OrderedDict([
            ('conv1', [self.conv1, relu]),
            ('pool1', [lambda x: max_pooling_2d(x, ksize=3, stride=2),
                       lambda x: local_response_normalization(x, n=5)]),
            ('conv2_reduce', [self.conv2_reduce, relu]),
            ('conv2', [self.conv2, relu]),
            ('pool2', [lambda x: local_response_normalization(x, n=5),
                       lambda x: max_pooling_2d(x, ksize=3, stride=2)]),
            ('inc3a', [self.inc3a]),
            ('inc3b', [self.inc3b]),
            ('pool3', [lambda x: max_pooling_2d(x, ksize=3, stride=2)]),
            ('inc4a', [self.inc4a]),

            ('inc4b', [self.inc4b]),
            ('inc4c', [self.inc4c]),
            ('inc4d', [self.inc4d]),
            ('inc4e', [self.inc4e]),
            ('pool4', [lambda x: max_pooling_2d(x, ksize=3, stride=2)]),
            ('inc5a', [self.inc5a]),
            ('inc5b', [self.inc5b]),
            ('pool6', [lambda x: average_pooling_2d(x, ksize=7, stride=1)]),
            ('prob', [lambda x: dropout(x, ratio=0.4),
                      self.loss3_fc])
        ])

    @classmethod
    def convert_caffemodel_to_npz(cls, path_caffemodel, path_npz):
        """Converts a pre-trained caffemodel to a chainer model.

        Args:
            path_caffemodel (str): Path of the pre-trained caffemodel.
            path_npz (str): Path of the converted chainer model.
        """

        # As CaffeFunction uses shortcut symbols,
        # we import CaffeFunction here.
        from chainer.links.caffe.caffe_function import CaffeFunction
        caffemodel = CaffeFunction(path_caffemodel)
        chainermodel = cls(pretrained_model=None)
        _transfer_googlenet(caffemodel, chainermodel)
        npz.save_npz(path_npz, chainermodel, compression=False)

    def __call__(self, x, layers=['prob'], test=True):
        """Computes all the feature maps specified by ``layers``.

        Args:
            x (~chainer.Variable): Input variable.
            layers (list of str): The list of layer names you want to extract.
            test (bool): If ``True``, dropout runs in test mode.

        Returns:
            Dictionary of ~chainer.Variable: A directory in which
            the key contains the layer name and the value contains
            the corresponding feature map variable.

        """

        h = x
        activations = {}
        target_layers = set(layers)
        for key, funcs in self.functions.items():
            if len(target_layers) == 0:
                break
            for func in funcs:
                if func is dropout:
                    h = func(h, train=not test)
                else:
                    h = func(h)
            if key in target_layers:
                activations[key] = h
                target_layers.remove(key)
        return activations


def _transfer_Wb(src, dst):
    # transfer weights for Convolution and Linear layers
    dst.W.data[:] = src.W.data
    dst.b.data[:] = src.b.data


def _transfer_inception(src, dst, inception_name):
    _transfer_Wb(src[inception_name + '/' + '1x1'], dst.conv1)
    _transfer_Wb(src[inception_name + '/' + '3x3'], dst.conv3)
    _transfer_Wb(src[inception_name + '/' + '5x5'], dst.conv5)
    _transfer_Wb(src[inception_name + '/' + '3x3_reduce'], dst.proj3)
    _transfer_Wb(src[inception_name + '/' + '5x5_reduce'], dst.proj5)
    _transfer_Wb(src[inception_name + '/' + 'pool_proj'], dst.projp)


def _transfer_googlenet(src, dst):
    _transfer_Wb(src['conv1/7x7_s2'], dst.conv1)
    _transfer_Wb(src['conv2/3x3_reduce'], dst.conv2_reduce)
    _transfer_Wb(src['conv2/3x3'], dst.conv2)

    _transfer_inception(src, dst.inc3a, 'inception_3a')
    _transfer_inception(src, dst.inc3b, 'inception_3b')
    _transfer_inception(src, dst.inc4a, 'inception_4a')
    _transfer_inception(src, dst.inc4b, 'inception_4b')
    _transfer_inception(src, dst.inc4c, 'inception_4c')
    _transfer_inception(src, dst.inc4d, 'inception_4d')
    _transfer_inception(src, dst.inc4e, 'inception_4e')
    _transfer_inception(src, dst.inc5a, 'inception_5a')
    _transfer_inception(src, dst.inc5b, 'inception_5b')

    _transfer_Wb(src['loss3/classifier'], dst.loss3_fc)
    _transfer_Wb(src['loss1/conv'], dst.loss1_conv)
    _transfer_Wb(src['loss1/fc'], dst.loss1_fc1)
    _transfer_Wb(src['loss1/classifier'], dst.loss1_fc2)
    _transfer_Wb(src['loss2/conv'], dst.loss2_conv)
    _transfer_Wb(src['loss2/fc'], dst.loss2_fc1)
    _transfer_Wb(src['loss2/classifier'], dst.loss2_fc2)


def _make_npz(path_npz, url, model):
    path_caffemodel = download.cached_download(url)
    print('Now loading caffemodel (usually it may take few minutes)')
    GoogLeNet.convert_caffemodel_to_npz(path_caffemodel, path_npz)
    npz.load_npz(path_npz, model)
    return model


def _retrieve(name, url, model):
    root = download.get_dataset_directory('pfnet/chainer/models/')
    path = os.path.join(root, name)
    return download.cache_or_load_file(
        path, lambda path: _make_npz(path, url, model),
        lambda path: npz.load_npz(path, model))



import numpy as np
import os.path as osp
import warnings

from skimage.io import imsave

try:
    import tensorflow as tf
    from tensorflow.contrib.tensorboard.plugins import projector
    _available = True

except ImportError:
    _available = False


def _check_available():
    if not _available:
        warnings.warn('Tensorflow is not installed on your environment, '
                      'so a function embedding_tensorboard can not be used.'
                      'Please install tensorflow.\n\n'
                      '  $ pip install tensorflow\n')


def embedding_tensorboard(features, images=None, labels=None,
                          log_dir='/tmp/chainer_cv/',
                          embedding_variable_name='embedding'):
    """Save files for embedding visualization on Tensorboard.

    After calling this function, you can visualize embedding by opening up
    Tensorboard with the following command.

    `tensorboard --logdir=log_dir`

    Note that you need to install tensorflow to use this function.

    Args:
        features (numpy.ndarray): An array of shape `(N, D)`. `N` is the
            number of features to visualize and `D` is the dimension of the
            feature vector.
        images (numpy.ndarray): An array of shape `(N, H, W, 3)` or
            `(N, H, W)`. This is thumbnail images. `N` is equal to
            the number of features.
        labels (dict of list): The key is the name of attribute. The value is
            a list whose length equals the number of features.
            (ex. `{'id': [1,2], 'label': ['cat', 'dog']})

    .. seealso::
        https://www.tensorflow.org/get_started/embedding_viz

    """
    _check_available()
    sess = tf.Session()

    # The embedding variable, which needs to be stored
    embedding_var = tf.Variable(features, name=embedding_variable_name)
    sess.run(embedding_var.initializer)
    summary_writer = tf.summary.FileWriter(log_dir)
    config = projector.ProjectorConfig()
    embedding = config.embeddings.add()
    embedding.tensor_name = embedding_var.name

    if labels is not None:
        embedding.metadata_path = osp.join(log_dir, 'metadata.tsv')
        _create_metadata(labels, embedding.metadata_path)

    if images is not None:
        assert images.shape[0] == features.shape[0]
        embedding.sprite.image_path = osp.join(log_dir, 'sprite.png')
        H, W = images.shape[1], images.shape[2]
        embedding.sprite.single_image_dim.extend([H, W])
        sprite = _images_to_sprite(images)
        imsave(embedding.sprite.image_path, sprite)

    projector.visualize_embeddings(summary_writer, config)
    saver = tf.train.Saver([embedding_var])
    saver.save(sess, osp.join(log_dir, 'embedding.ckpt'), 1)


def _images_to_sprite(data):
    """Creates the sprite image along with any necessary padding

    Args:
        data: (N, H, W, 3) image

    Returns:
        data: Properly shaped H'xW'x3 image with any necessary padding.
    """
    if len(data.shape) == 3:
        data = np.tile(data[..., np.newaxis], (1, 1, 1, 3))
    data = data.astype(np.float32)
    min = np.min(data.reshape((data.shape[0], -1)), axis=1)
    data = (data.transpose(1, 2, 3, 0) - min).transpose(3, 0, 1, 2)
    max = np.max(data.reshape((data.shape[0], -1)), axis=1)
    data = (data.transpose(1, 2, 3, 0) / max).transpose(3, 0, 1, 2)

    n = int(np.ceil(np.sqrt(data.shape[0])))
    padding = ((0, n ** 2 - data.shape[0]), (0, 0),
               (0, 0)) + ((0, 0),) * (data.ndim - 3)
    data = np.pad(data, padding, mode='constant',
                  constant_values=0)
    # Tile the individual thumbnails into an image.
    data = data.reshape(
        (n, n) + data.shape[1:]).transpose(
            (0, 2, 1, 3) + tuple(range(4, data.ndim + 1)))
    data = data.reshape(
        (n * data.shape[1], n * data.shape[3]) + data.shape[4:])
    data = (data * 255).astype(np.uint8)
    return data


def _create_metadata(labels, metadata_file):
    """Record metadata to metadata_file

    Args:
        labels: each keyword contains list of values.
            (ex. `{'a': [1,2,3], 'b': [10, 11, 12]}`)
    """
    keys = labels.keys()
    values = zip(*labels.values())

    with open(metadata_file, 'w') as f:
        for key in keys:
            f.write('{}\t'.format(key))
        f.write('\n')
        for value in values:
            for elem in value:
                f.write('{}\t'.format(elem))
            f.write('\n')


if __name__ == '__main__':
    emb = np.random.uniform(size=(1000, 4096))
    N = emb.shape[0]
    H = 32
    W = 32
    img = np.random.uniform(low=0, high=255, size=(N, H, W, 3))
    labels = {'label': range(N)}

    embedding_tensorboard(emb, img, labels)



#!/usr/bin/env python

from distutils.core import setup
from setuptools import find_packages

description = """
Collection of Deep Learning Computer Vision Algorithms implemented in Chainer
"""


setup(
    name='chainer_cv',
    version='0.2',
    packages=find_packages(),
    author='Yusuke Niitani',
    author_email='yuyuniitani@gmail.com',
    license='MIT',
    description=description,
    install_requires=open('requirements.txt').readlines(),
)



import data_augmentation  # NOQA
import preprocessing  # NOQA
import cache_dataset  # NOQA

from dataset_wrapper import DatasetWrapper  # NOQA
from cache_dataset.cache_dataset_wrapper import CacheDatasetWrapper  # NOQA
from cache_dataset.cache_array_dataset_wrapper import CacheArrayDatasetWrapper  # NOQA
from preprocessing.pad_wrapper import PadWrapper  # NOQA
from preprocessing.subtract_wrapper import SubtractWrapper  # NOQA
from preprocessing.resize_wrapper import ResizeWrapper  # NOQA
from data_augmentation.crop_wrapper import CropWrapper  # NOQA
from data_augmentation.random_mirror_wrapper import RandomMirrorWrapper  # NOQA
from split_dataset.keep_subset_wrapper import KeepSubsetWrapper  # NOQA



from semantic_segmentation.semantic_segmentation_vis_out import SemanticSegmentationVisOut  # NOQA
from embed_images import EmbedImages  # NOQA
from image_retrieval.measure_k_recall import MeasureKRetrieval  # NOQA



import copy
import numpy as np
import os.path as osp
import warnings

import chainer
from chainer.dataset import convert
from chainer import reporter

try:
    import sklearn.neighbors
    _available = True

except ImportError:
    _available = False


def _check_available():
    if not _available:
        warnings.warn('scikit-learn is not installed on your environment, '
                      'so an extension MeasureKRetrieval can not be used.'
                      'Please install scikit-learn.\n\n'
                      '  $ pip install scikit-learn\n')


class MeasureKRetrieval(chainer.training.extension.Extension):

    """Measure Recall@K score for image retrieval task.

    The element in the second index of the returned tuple will be class id.

    Args:
        iterator (chainer.iterators.Iterator): Dataset iterator for the
            images to be embedded. The element in the second index of the
            returned tuple will be class id.
        features_file (string): A file where embedded features are saved.
        ks (list of ints)
        window_size (int)
        n_jobs (int): number of processes run in parallel to compute the
            k-nearest neighbor.
    """

    invoke_before_training = False
    priority = chainer.training.extension.PRIORITY_WRITER

    def __init__(self, iterator, features_file, ks,
                 window_size=10000, n_jobs=6):
        _check_available()
        if not _available:
            return

        self.iterator = iterator
        self.features_file = features_file
        self.ks = ks

        self.window_size = window_size

        n_jobs = n_jobs
        self.max_k = np.max(ks)
        self.nbrs = sklearn.neighbors.NearestNeighbors(n_jobs=n_jobs)

    def __call__(self, trainer):
        if not _available:
            return
        features_file = osp.join(trainer.out, self.features_file)

        iterator = copy.copy(self.iterator)
        features = np.load(features_file)
        optimizer = trainer.updater.get_optimizer('main')

        classes = []
        for v in iterator:
            arrays = convert.concat_examples(v)
            classes.append(arrays[1])
        classes = np.concatenate(classes, axis=0)

        if features.shape[0] != classes.shape[0]:
            raise ValueError(
                'batch size of features and the class array differ')

        n = features.shape[0]
        n_match = {k: [] for k in self.ks}

        self.nbrs.fit(features)
        for i in range(0, n, self.window_size):
            the_slice = slice(
                i * self.window_size, (i + 1) * self.window_size)
            src_features = features[the_slice]
            src_classes = classes[the_slice]

            indices = self.nbrs.kneighbors(
                src_features, n_neighbors=self.max_k + 1,
                return_distance=False)
            indices = indices[:, 1:]
            knbr_classes = classes[indices]  # (window, max_k)

            match = knbr_classes == src_classes[:, None]

            for k in self.ks:
                n_match_k = np.any(match[:, :k], axis=1)
                n_match[k].append(n_match_k)

        for k in self.ks:
            n_match[k] = np.concatenate(n_match[k])

            reporter.report({'recall@{}'.format(k): np.mean(n_match[k])},
                            optimizer.target)



import numpy as np

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


def keep_subset(func, indices):

    def wrapped(i):
        wrapped_i = indices[i]
        return func(wrapped_i)
    return wrapped


class KeepSubsetWrapper(DatasetWrapper):

    """Keep subset of dataset.

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped
        indices (list of int): inices of the examples that are used after
            wrapping the dataset. The order of examples from the wrapped
            dataset corresponds to the ordering of ints in `indices`.
        wrapped_func_names (list of strings): List of the name of functions
            which you will be affected by selecting subset of all indices.
            These functions include `get_example` and similar functions that
            take (self, i) as its argument.
            If `wrapped_func_names` is `None`, this arguement will be
            `[\'get_example\']`.

    """

    def __init__(self, dataset, indices, wrapped_func_names=None):
        super(KeepSubsetWrapper, self).__init__(dataset)

        self._indices = indices
        if (np.max(self._indices) > len(self._dataset) or
                np.min(self._indices) < 0):
            raise ValueError('indices need to be a valid index to a dataset')

        if wrapped_func_names is None:
            wrapped_func_names = ['get_example']

        for name in wrapped_func_names:
            wrapped_func = keep_subset(
                getattr(self, name), self._indices)
            setattr(self, name, wrapped_func)

    def __len__(self):
        return len(self._indices)



import numpy as np

from chainer_cv.wrappers.dataset_wrapper import DatasetWrapper


vgg_subtract_bgr = np.array([103.939, 116.779, 123.68], np.float32)


class SubtractWrapper(DatasetWrapper):

    """Subtract images by constants

    Subtract values from the first array returned by the wrapped dataset.

    Args:
        dataset: a chainer.dataset.DatasetMixin to be wrapped
        value (float or array-like): constant to subtract data from. This is
            either 0 or 1 dimensional array. The value is subtracted along
            channel axis. Default is the constant value used to subtract
            from image when preparing for computer vision networks like VGG.

    """

    def __init__(self, dataset, value=vgg_subtract_bgr):
        super(SubtractWrapper, self).__init__(dataset)
        if not isinstance(value, np.ndarray):
            value = np.array(value)
        # convert to CHW format
        if value.ndim == 0:
            value = value[None, None, None]
        elif value.ndim == 1:
            value = value[:, None, None]
        elif value.ndim != 3:
            raise ValueError('only accept 0d, 1d or 3d array')

        self.value = value

    def _get_example(self, in_data):
        x = in_data[0]
        x -= self.value
        return (x,) + in_data[1:]
